/*******************************************************************************
*  Project Name: Eg1_ADC_DMA_DAC
*  Device Tested: CY8C3866AXI, CY8C5868AXI
*  Software Version: PSoC Creator 3.0 SP1
*  Complier tested: Keil(C51) and GCC
*  Related Hardware: CY8CKIT-001, CY8CKIT-030 and CY8CKIT-050
********************************************************************************
********************************************************************************
* Copyright (2014)), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/

/****************************************************************************** 
					PROJECT DESCRIPTION 
******************************************************************************* 
 * The project aims at explaining how to use DMA for a simple transfer data from ADC to DAC
 * The DMA transfers 8 bit data from the ADC output register to the VDAC data register. 
 * Thus the input signal given to the ADC will be reproduced at the DAC output.
 * In this example, the ADC is limited to 0 to 1.024 V input range. The VDAC is also limited to 0 to 1.024 V.
 * To observe the output, a signal should be given from the function generator to the ADC and the output
 * can be observed using an oscilloscope.
 * In this example the ADC sampling frequency(fs) is 384kHZ and the input is reconstructed best 
 * if the input frequency is less than or equal to ~84kHz because the delta sigma ADCs have a
 * low pass nature with -3dB frequency of 0.22fs. 
 *
 * Hardware connections:
 * Connect the Function generator to pin P0[2] - This is the input to ADC
 * Connect the Oscilloscope probe to pin P0[0] - This is the VDAC output
 ******************************************************************************/

#include <device.h>

/* Variable declarations for DMA . 
* These variables are defined as global variables to avoid "may be used before being set" warning 
* issued by the PSoC 5 compilers  MDK/RVDS.In this case these variables are automatically initialized to zero */

/* DMA_Chan is used to store the DMA channel handle */ 
uint8 DMA_Chan;

/* DMA_TD array is used to store all of the TDs associated with the channel
 * Since there is only one TD in this example, DMA_TD array contains only one element */
uint8 DMA_TD[1];

void main(void)
{

    /* Define for DMA Configuration */
    #define DMA_BYTES_PER_BURST 1
    #define DMA_REQUEST_PER_BURST 1
	
	/* Both source and destinations base address is peripheral base address */
    #define DMA_SRC_BASE (CYDEV_PERIPH_BASE)
    #define DMA_DST_BASE (CYDEV_PERIPH_BASE)

    /* Initialization of DMA */

	/* Step 1: DmaInitialize - Initialize the DMA channel
	 * Burst count = 1, (8 bit data transferred from ADC to VDAC at a time )
	 * Request per bit = 1 (Every burst transfer requires a new request)
	 * High byte of source address = Upper 16 bits of ADC data register
     * High byte of destination address =  Upper bytes of the VDAC8 data register
     * DMA_Chan holds the channel handle returned by the ‘DmaInitialize’ function. This is 
	 * used for all further references of the channel */

    DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, 
                                 HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
                                 
                                 
    /* Step 2: Allocate TD */
    DMA_TD[0] = CyDmaTdAllocate();
   
    /* Step 3: CyDmaTdSetConfiguration - Configures the TD:
	 * Transfer count = 1 (Total number of bytes to transfer from the ADC to DAC)
	 * Next Td = DMA_TD[0]. The same td has to repeat itself 
	 * Configuration = No configuration required
	*/
    CyDmaTdSetConfiguration(DMA_TD[0], 1, DMA_TD[0], 0);
    
    /* Step 4: CyDmaTdSetAddress - Configure the lower 16 bit source and destination addresses 
	 * Source address = Lower 16 bits of ADC data register
	 * Destination address = Lower 16 bits of VDAC8 data register */
    CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)VDAC8_Data_PTR));
    
    /* Step 5: Map the TD to the DMA Channel */
    CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
    
    /* Step 6: Enable the channel
	 * The Channel is enabled with preserveTds set to 1. This will reload the original TD settings 
	 * after the transfer is complete so that TD can be repeated */
    CyDmaChEnable(DMA_Chan, 1);

	/* Start ADC and DAC */
	ADC_DelSig_Start();
	VDAC8_Start();
	
	/* Start ADC Conversion */
	ADC_DelSig_StartConvert();
	
	for(;;)
    {
        /* Your code here. */
        /* The ADC data is moved to the DAC by the DMA in the background.
         * No CPU intervention is required in this project */

    }
}

/* [] END OF FILE */
