/*******************************************************************************
*  Project Name: Eg4_Mem_DMA_Mem
*  Device Tested: CY8C3866AXI, CY8C5868AXI
*  Software Version: PSoC Creator 3.0 SP1
*  Complier tested: Keil(C51) and GCC
*  Related Hardware: CY8CKIT-001, CY8CKIT-030 and CY8CKIT-050
********************************************************************************
********************************************************************************
* Copyright (2014)), Cypress Semiconductor Corporation. All Rights Reserved.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
******************************************************************************************/

/******************************************************************************************
					PROJECT DESCRIPTION 
*******************************************************************************************  
* The project demonstrates how to copy data from memory to memory (flash to RAM) using DMA.
* The DMA transfers 8 bytes of data from a flash array to a RAM array on CPU request. 
* The RAM array contents are then displayed on the LCD.
    
*******************************************************************************************/

#include <device.h>

/*  The source data array is stored in flash. The declaration for the source array will change based on PSoC3 or PSoC5.
    * For PSoC3 keil compiler the source array can be put in flash using the keyword "code". But this doesn't work for 
    * PSoC5. For PSoC5 GCC compiler the keyword "const" is used for declaring source array. In order to make the code 
    * compatible with both PSoC3 and PSoC5, the source array declaration is done within preprocessor statement 
    * which checks for the compiler used. PSoC3 uses Keil_C51 and it is defined using __C51__
    * Else declaration for PSoC5 is used.
*/


/* Flag to indicate DMA transfer is complete */
volatile uint8 flag_DMADone;

/* Number of data to be transferred */
#define DATASIZE 8

/* Variable declarations for DMA . 
* These variables are defined as global variables to avoid "may be used before being set" warning 
* issued by the PSoC 5 compilers  MDK/RVDS.In this case these variables are automatically initialized to zero */

/* DMA_Chan is used to store the DMA channel handle */ 
uint8 DMA_Chan;

/* DMA_TD array is used to store all of the TDs associated with the channel
 * Since there is only one TD in this example, DMA_TD array contains only one element */
uint8 DMA_TD[1];


/*An 8 byte array which stores values 0 to 7 is declared in flash and is the source of DMa transfer. 
* The 'CYCODE const' specifies that the array should be placed in flash*/
CYCODE const uint8 sourceArray[8] = {0, 1, 2, 3, 4, 5, 6, 7};

/* The destination array stored in SRAM to receive data */
uint8 destinationArray[8] = {0};
 
/* DMA_DISABLE_TD definition not available for PSoC 5LP inside CyDmac.h in PSoC Creator 2.1 SP1.
 * So, manually define the DMA_DISABLE_TD */
#if (CY_PSOC5LP)
	#if !defined(DMA_DISABLE_TD)
	    #define DMA_DISABLE_TD          0xFEu
	#endif
#endif  /* (CY_PSOC5LP) */

void main()
{

	/* Variable used for display index */
	uint8 index;
	
     
    /* Step1 : DmaInitialize - Initialize the DMA 
	 * Burst count (Bytes per burst) = 1 byte. Transfer data byte by byte.
	 * Request per burst = 0. Every burst after the first burst are automatically done without a separate DMA request.
	 * Upper source address = HI16(CYDEV_FLS_BASE) for PSoC 3
							= HI16(sourceArray)for PSoC 5 LP
	 * Upper destination address = HI16(CYDEV_SRAM_BASE) for PSoC 3
								 = HI16(destinationArray)for PSoC 5 LP
	 * DMA_Chan holds the channel handle returned by the ‘DmaInitialize’ function. This is 
	 * used for all further references of the channel 
	 * The parameter upper 16 bits of the source and destination address is specific to the device family.
	 * Hence the DMA Initialization will change based on PSoC3 or PSoC5.
	 * In order to make the code compatible with both PSoC3 and PSoC5, the DMA
     * initialization is done within preprocessor statement which checks for the compiler used.
     * PSoC3 uses Keil_C51 and it is defined using __C51__
     * else initialization for PSoC5 is used */
	#if (defined(__C51__))
	    DMA_Chan = DMA_DmaInitialize(1, 0, HI16(CYDEV_FLS_BASE), HI16(CYDEV_SRAM_BASE));
    
    #else
        DMA_Chan = DMA_DmaInitialize(1, 0, HI16(sourceArray), HI16(destinationArray) );
    #endif
    
    /* Step2 :CyDmaTdAllocate - Allocate TD */
	DMA_TD[0] = CyDmaTdAllocate();

	/* Step3 :CyDmaTdSetConfiguration - Configures the TD:
	 * tdHandle = DMA_TD[0] - TD handle previously returned by CyDmaTdAlloc()
	 * Set transfer count as 8 to transfer 8 bytes of data from flash to SRAM.
     * Next Td = DMA_INVALID_TD , disable the channel after the transfer is finished
	 * TD configuration = increment source and destination addresses after each transaction
     * and enable the Terminal Output so as to trigger ISR_DMADone_Interrupt when transfer is complete */
	CyDmaTdSetConfiguration(DMA_TD[0], DATASIZE, DMA_DISABLE_TD, TD_INC_SRC_ADR | TD_INC_DST_ADR | DMA__TD_TERMOUT_EN);
	
    /* Step 4 : CyDmaTdSetAddress - Configure the lower 16 bit source and destination addresses 
	 * Set the source address as sourceArray and destination address as destination Array */
	CyDmaTdSetAddress(DMA_TD[0], (uint16)((uint32)sourceArray), (uint16)((uint32)destinationArray) );
	
    /* Step 5: Map the TD to the DMA Channel */
	CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
	
    /* Step 6: Enable the DMA channel */
	CyDmaChEnable(DMA_Chan, 1);
	
    /* Start LCD and enable all interrupts */
    LCD_Start();
	
	/* Enable Interrupts */
	ISR_DMADone_Start();
    CYGlobalIntEnable;
	
    /* Display the destination array contents before the data transfer */
	LCD_Position(0, 0);
    
	for(index = 0; index < DATASIZE; index++)
    {
	    LCD_PrintInt8(destinationArray[index]);
    }
	/* Add 1 second delay before triggering the DMA channel */
	CyDelay(1000);
	
    /* Trigger DMA channel using CPU */
	CyDmaChSetRequest(DMA_Chan, CPU_REQ);	
	
    /* Wait till DMA transfer is complete 
	 * This flag is set in the 'ISR_DMADone' once the DMA transfer is complete */
	while(flag_DMADone == 0);
	
	/* Clear LCD display*/
    LCD_ClearDisplay();
    
    /* Display the new RAM array contents on LCD */
	LCD_Position(0,0);
    
	for(index = 0; index < DATASIZE; index++)
    {
	    LCD_PrintInt8(destinationArray[index]);
    }
    
    /* Display message to show transfer is complete*/
    LCD_Position(1, 0);
	LCD_PrintString("TRANSFERRED");
	
    for(;;)
    {

	}
}

/* [] END OF FILE */
