/********************************************************************************
*  Project Name: Eg5_TD_Chaining
*  Device Tested: CY8C3866AXI, CY8C5868AXI
*  Software Version: PSoC Creator 3.0 SP1
*  Complier tested: Keil(C51) and GCC
*  Related Hardware: CY8CKIT-001, CY8CKIT-030 and CY8CKIT-050
*********************************************************************************
* Copyright (2014)), Cypress Semiconductor Corporation. All Rights Reserved.
*********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*******************************************************************************/

/****************************************************************************** 
					PROJECT DESCRIPTION 
******************************************************************************* 
* This example project demonstrates how to do ping pong buffering of 16 bit ADC data using DMA.
* The DMA is configured to buffer specified number of ADC samples on a switch press.
* The samples are filled in 'adc_sampleArray1' and 'adc_sampleArray2' .
* DMA buffers ADC samples in'adc_sampleArray1'first and then 'adc_sampleArray2' 
* The DMA channel is enabled on each switch press and disabled after collecting the specified number of ADC samples.
* Connect P6[1] to switch Sw1 on DVK and connect ADC input to P0[2]. 
* Run the debugger and Verify the output - Watch the 'adc_sampleArray1'and
* 'adc_sampleArray2' buffer in watch window of debugger.
******************************************************************************/

#include <device.h>

/* Define for the number of samples to be transferred to memory on each switch press */
#define NO_OF_SAMPLES1 50
#define NO_OF_SAMPLES2 20

#define TRANSFER_COUNT1 NO_OF_SAMPLES1 * 2
#define TRANSFER_COUNT2 NO_OF_SAMPLES2 * 2

#define DMA_BYTES_PER_BURST 2
#define DMA_REQUEST_PER_BURST 1
#define DMA_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_DST_BASE (CYDEV_SRAM_BASE)

volatile uint8 switch_flag;
volatile uint8 DMADone_flag;

/* Variable declarations for DMA . 
 * These variables are defined as global variables to avoid "may be used before being set" warning 
 * issued by the PSoC 5 compilers  MDK/RVDS.In this case these variables are automatically initialized to zero */

/* DMA_Chan is used to store the DMA channel handle */ 
uint8 DMA_Chan;

/* DMA_TD array is used to store all of the TDs associated with the channel
 * Since there are two TDs in this example, DMA_TD array contains two elements */
uint8 DMA_TD[2];

/* DMA_DISABLE_TD definition not available for PSoC 5LP inside CyDmac.h in PSoC Creator 2.1 SP1.
 * So, manually define the DMA_DISABLE_TD */
#if (CY_PSOC5LP)
	#if !defined(DMA_DISABLE_TD)
	    #define DMA_DISABLE_TD          0xFEu
	#endif
#endif  /* (CY_PSOC5LP) */

void main()
{
	uint16 adc_sampleArray1[NO_OF_SAMPLES1]={0};
	uint16 adc_sampleArray2[NO_OF_SAMPLES2]={0};

    /* Step 1: DmaInitialize - Initialize the DMA channel
	 * Burst count (Bytes per burst) = 2 bytes. Transfer 2-bytes from ADC to memory on each trigger.
	 * Request per burst = 1. Every burst transfer requires a separate request. 
	 * Upper source address = HI16(CYDEV_PERIPH_BASE)
	 * Upper destination address = HI16(CYDEV_SRAM_BASE)
     * DMA_Chan holds the channel handle returned by the ‘DmaInitialize’ function. This is 
	 * used for all further references of the channel 
	 */	
	 DMA_Chan = DMA_DmaInitialize(DMA_BYTES_PER_BURST, DMA_REQUEST_PER_BURST, HI16(DMA_SRC_BASE), HI16(DMA_DST_BASE));
		
    /* Step 2: CyDmaTdAllocate - Allocate TDs */
	DMA_TD[0] = CyDmaTdAllocate();
	DMA_TD[1] = CyDmaTdAllocate();

    /* Step 3: CyDmaTdSetConfiguration - Configures the TD */
	#if (defined(__C51__)) /*If PSoC 3 , use the TD conffigurations for PSoC 3*/
	/* tdHandle = DMA_TD[0] - TD handle previously returned by CyDmaTdAlloc()
	 * transferCount = NO_OF_SAMPLES2 * 2
	 * nextTd = DMA_TD[1], Chain the first TD to the second TD
	 * configuration = Increment Destination address after each burst transfer
	 *				   In PSoC 3, byte swap is enabled for endian swapping; this is not required for PSoC 5LP 
	 */
	CyDmaTdSetConfiguration(DMA_TD[0],TRANSFER_COUNT1, DMA_TD[1], TD_SWAP_EN | TD_INC_DST_ADR);
	
	/* tdHandle = DMA_TD[1] - TD handle for the second TD
	 * transferCount = NO_OF_SAMPLES2 * 2
	 * nextTd = DMA_DISABLE_TD, Disable the DMA channel after the transfer is complete
	 * configuration = Increment Destination address after each burst transfer
	 *				   Generate termout when the transfer is finished.
	 *				   In PSoC 3, byte swap is enabled for endian swapping; 
	 */
	CyDmaTdSetConfiguration(DMA_TD[1], TRANSFER_COUNT2, DMA_DISABLE_TD, TD_SWAP_EN | TD_INC_DST_ADR | DMA__TD_TERMOUT_EN);
	
	#else /* PSoC 5 LP */
	/* Byte swap is not required for PSoC 5 LP */
	CyDmaTdSetConfiguration(DMA_TD[0],TRANSFER_COUNT1, DMA_TD[1],  TD_INC_DST_ADR);
	CyDmaTdSetConfiguration(DMA_TD[1], TRANSFER_COUNT2, DMA_DISABLE_TD, TD_INC_DST_ADR | DMA__TD_TERMOUT_EN);
	#endif

    /* Step 4: CyDmaTdSetAddress - Configure the lower 16 bit source and destination addresses 
	 * Source address = Lower 16 bits of ADC data register
	 * Destination address = Lower 16 bits of 'ADC_sample' arrays */
	CyDmaTdSetAddress(DMA_TD[0], LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)&adc_sampleArray1[0]));
	CyDmaTdSetAddress(DMA_TD[1], LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)&adc_sampleArray2[0]));

    /* Step 5: Map the TD to the DMA Channel */
	CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);

	/* Start ADC */
	ADC_DelSig_Start();
    
    /* Disable the ADC ISR as it is not required */
    ADC_DelSig_IRQ_Disable();

	/* Start ADC conversion */
	ADC_DelSig_StartConvert();

	/* Start isr connected to collect samples pin */
	ISR_Switch_Start();
	
	/* Start isr connected to DMA nrq signal */
	ISR_DMA_Done_Start();

	/* Enable global interrupt */
	CYGlobalIntEnable; 

	for(;;)
    {
	    /* switch_flag is set in ISR_Switch when the switch is pressed */
		if(switch_flag)
		{
         
			/* Enable the DMA channel */
            CyDmaChSetInitialTd(DMA_Chan, DMA_TD[0]);
			CyDmaChEnable(DMA_Chan, 1);
			
			/* Clear the flag */
			switch_flag = 0;
			
		}/* If statement ends here */

	    /* DMADone_flag is set inside ISR_DMA_Done which is trigger
		 * after both the adc sample arrays are filled with ADC data */
		if(DMADone_flag)
		{
			/* Put a break point here to view the data in the 'adc_sampleArray1' and 
			 * 'adc_sampleArray2' in the debug watch window */
			DMADone_flag = 0;
			
		}/* If statement ends here */
	
    }/* for loop ends here */
        
	/* Place your application code here. */
}

/* [] END OF FILE */
